<?php

declare(strict_types=1);

namespace Qls\ServicePoints\Service;

use Qls\ServicePoints\Struct\ServicePoint;
use Qls\ServicePoints\Struct\ServicePointInformation;
use Shopware\Core\Checkout\Cart\Cart;
use Shopware\Core\Checkout\Cart\Delivery\Struct\ShippingLocation;
use Shopware\Core\Checkout\Shipping\ShippingMethodCollection;
use Shopware\Core\Checkout\Shipping\ShippingMethodEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;

class ShippingMethodService
{
    private const DEFAULT_URL = 'https://embed.pakketdienstqls.nl';
    private const DEFAULT_COUNTRY = 'NL';
    private const DEFAULT_ZIPCODE = '3313LC';
    private const SERVICE_POINT_INFORMATION = 'servicePointInformation';

    private EntityRepositoryInterface $ruleRepository;

    private EntityRepositoryInterface $deliveryTimeRepository;

    private EntityRepositoryInterface $shippingMethodRepository;

    private SystemConfigService $systemConfigService;

    public function __construct(EntityRepositoryInterface $ruleRepository, EntityRepositoryInterface $deliveryTimeRepository, EntityRepositoryInterface $shippingMethodRepository, SystemConfigService $systemConfigService)
    {
        $this->ruleRepository = $ruleRepository;
        $this->deliveryTimeRepository = $deliveryTimeRepository;
        $this->shippingMethodRepository = $shippingMethodRepository;
        $this->systemConfigService = $systemConfigService;
    }

    private function getServicePointTranslations(): array
    {
        return [
            'de-DE' => [
                'name' => 'Service Point Zustellung',
            ],
            'en-GB' => [
                'name' => 'Service Point Delivery',
            ],
            'es-ES' => [
                'name' => 'Punto de servicio de entrega',
            ],
            'fr-BE' => [
                'name' => 'Livraison en Point Relais',
            ],
            'fr-FR' => [
                'name' => 'Livraison en Point Relais',
            ],
            'nl-BE' => [
                'name' => 'Service Punt Levering',
            ],
            'nl-NL' => [
                'name' => 'Service Punt Levering',
            ],
        ];
    }

    public function createServicePointShippingMethod(Context $context): void
    {
        $rule = $this->ruleRepository->search(new Criteria(), $context)->first();
        $deliveryTime = $this->deliveryTimeRepository->search(new Criteria(), $context)->first();

        $data = [
            'translations' => $this->getServicePointTranslations(),
            'active' => false,
            'availabilityRuleId' => $rule->getId(),
            'deliveryTimeId' => $deliveryTime->getId(),
        ];

        $entityIds = $this->shippingMethodRepository
            ->create([$data], $context)
            ->getEventByEntityName('shipping_method')
            ->getIds();

        $this->systemConfigService->storeServicePointSystemConfig(reset($entityIds), $context);
    }

    private function getServicePointInformation(?ShippingLocation $shippingLocation): ServicePointInformation
    {
        return new ServicePointInformation(
            true,
            $this->getServicePointShippingMethodEndpoint(
                $this->getCountry($shippingLocation),
                $this->getPostalCode($shippingLocation),
            ),
            $this->getServicePoint($shippingLocation)
        );
    }

    private function getServicePoint(?ShippingLocation $shippingLocation): ?ServicePoint
    {
        if (!$shippingLocation) {
            return null;
        }

        if (!$address = $shippingLocation->getAddress()) {
            return null;
        }

        if (!$customFields = $address->getCustomFields()) {
            return null;
        }

        if (!in_array(ServicePoint::CODE, array_keys($customFields))) {
            return null;
        }

        return ServicePoint::fromArray($customFields);
    }

    private function getShippingLocation(Cart $cart): ?ShippingLocation
    {
        return ($delivery = $cart->getDeliveries()->first()) ? $delivery->getLocation() : null;
    }

    private function getCountry(?ShippingLocation $shippingLocation): string
    {
        if (!$shippingLocation) {
            return self::DEFAULT_COUNTRY;
        }
        return $shippingLocation->getCountry()->getIso();
    }

    private function getPostalCode(?ShippingLocation $shippingLocation): string
    {
        if (!$shippingLocation) {
            return self::DEFAULT_ZIPCODE;
        }
        return $shippingLocation->getAddress()->getZipcode();
    }

    private function getServicePointShippingMethodEndpoint(string $country, string $postalCode): string
    {
        return sprintf('%s/servicepoint/locator/%s/#postalcode=%s', self::DEFAULT_URL, $country, $postalCode);
    }

    public function addExtensionsToServicePointShippingMethod(ShippingMethodCollection $shippingMethodCollection, Cart $cart, Context $context): void
    {
        $servicePointShippingMethodId = $this->systemConfigService
            ->getServicePointSystemConfig($context)
            ->getConfigurationValue();

        $shippingMethodCollection
            ->filter(fn (ShippingMethodEntity $shippingMethodEntity) => $shippingMethodEntity->getId() === $servicePointShippingMethodId)
            ->map(function (ShippingMethodEntity $shippingMethodEntity) use ($cart) {
                $servicePointInformation = $this->getServicePointInformation($this->getShippingLocation($cart));

                $shippingMethodEntity->addExtension(self::SERVICE_POINT_INFORMATION, $servicePointInformation);
                $shippingMethodEntity->jsonSerialize();
            });
    }
}
