<?php

declare(strict_types=1);

namespace Qls\ServicePoints\Subscriber;

use Qls\ServicePoints\Service\CustomerAddressService;
use Qls\ServicePoints\Service\OrderAddressService;
use Qls\ServicePoints\Service\OrderService;
use Qls\ServicePoints\Service\ShippingMethodService;
use Qls\ServicePoints\Service\SystemConfigService;
use Qls\ServicePoints\Struct\ServicePoint;
use Shopware\Core\Checkout\Cart\Event\CheckoutOrderPlacedEvent;
use Shopware\Core\Checkout\Customer\CustomerEvents;
use Shopware\Core\Checkout\Order\Aggregate\OrderAddress\OrderAddressEntity;
use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityWrittenEvent;
use Shopware\Core\System\SalesChannel\Event\SalesChannelContextSwitchEvent;
use Shopware\Storefront\Page\Checkout\Confirm\CheckoutConfirmPageLoadedEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class CheckoutSubscriber implements EventSubscriberInterface
{
    private ShippingMethodService $shippingMethodService;

    private CustomerAddressService $customerAddressService;

    private OrderAddressService $orderAddressService;

    private OrderService $orderService;

    private SystemConfigService $systemConfigService;

    public function __construct(
        ShippingMethodService $shippingMethodService,
        CustomerAddressService $customerAddressService,
        OrderAddressService $orderAddressService,
        OrderService $orderService,
        SystemConfigService $systemConfigService
    ) {
        $this->shippingMethodService = $shippingMethodService;
        $this->customerAddressService = $customerAddressService;
        $this->orderAddressService = $orderAddressService;
        $this->orderService = $orderService;
        $this->systemConfigService = $systemConfigService;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            CheckoutConfirmPageLoadedEvent::class => 'onCheckoutConfirmPageLoaded',
            SalesChannelContextSwitchEvent::class => 'onSalesChannelContextSwitch',
            CheckoutOrderPlacedEvent::class => 'onCheckoutOrderPlaced',
            CustomerEvents::CUSTOMER_ADDRESS_WRITTEN_EVENT => 'onCustomerAddressChange',
        ];
    }

    public function onCheckoutConfirmPageLoaded(CheckoutConfirmPageLoadedEvent $event): void
    {
        $this->shippingMethodService->addExtensionsToServicePointShippingMethod(
            $event->getPage()->getShippingMethods(),
            $event->getPage()->getCart(),
            $event->getContext()
        );
    }

    /**
     * Unfortunately, the order is not yet available at this point. Therefore, the service point data is temporarily
     * attached to the customer's shipping address so that it is available later when the order is created.
     */
    public function onSalesChannelContextSwitch(SalesChannelContextSwitchEvent $event): void
    {
        $servicePointShippingMethodId = $this->systemConfigService->getServicePointSystemConfig($event->getContext())->getConfigurationValue();
        $newShippingMethodId = $event->getRequestDataBag()->get('shippingMethodId');

        if (
            $newShippingMethodId !== $servicePointShippingMethodId
            && null !== $event->getSalesChannelContext()->getShippingLocation()->getAddress()
        ) {
            $this->customerAddressService->removeServicePointFromCustomFields(
                $event->getSalesChannelContext()->getShippingLocation()->getAddress(),
                $event->getContext(),
            );
        }
        if ($newShippingMethodId === $servicePointShippingMethodId && $servicePoint = $event->getRequestDataBag()->get('servicePoint')) {
            $this->customerAddressService->addServicePointToCustomFields(
                ServicePoint::fromArray($servicePoint->all()),
                $event->getSalesChannelContext()->getShippingLocation()->getAddress(),
                $event->getContext(),
            );
        }
    }

    /**
     * At this stage, the order is available. That is why all necessary service point data is transferred from the
     * order address to the order.
     */
    public function onCheckoutOrderPlaced(CheckoutOrderPlacedEvent $event): void
    {
        $servicePoint = $event
            ->getOrder()
            ->getAddresses()
            ->reduce(function (?ServicePoint $servicePoint, OrderAddressEntity $orderAddressEntity) use ($event) {
                return $this->orderAddressService->getServicePointFromCustomFields($orderAddressEntity) ?? $servicePoint;
            });

        if ($servicePoint) {
            $this->orderService->addServicePointToCustomFields(
                $servicePoint,
                $event->getOrder(),
                $event->getContext(),
            );
        }
    }

    public function onCustomerAddressChange(EntityWrittenEvent $event): void
    {
        $payloads = $event->getPayloads();

        foreach ($payloads as $payload) {
            if (isset($payload['customFields'])) {
                return;
            }
            $this->customerAddressService->ensureServicePointIsRemovedFromCustomFields(
                $payload['id'],
                $event->getContext(),
            );
        }
    }
}
